"""
指定した『SS7』の積算の計算結果を利用して結果セットごとの相違をグラフ表示します。

Copyright (C) 2024 UNION SYSTEM Inc.
"""

import os
import sys
from typing import IO
import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt
from Python import Ss7Python as ss7py


def create_res_csv() -> None:
    """
    物件データフォルダ内に積算の結果CSVファイルを作成します。
    対象は全結果データです。（積算が未解析なら何もしない）
    Note:
        作成する結果CSVファイル名 cost1.csv(結果1) ～ cost5.csv(結果5)
    """
    # 出力項目名
    table_names: str = "17.4.2.1.部位別集計表;17.4.3.1.部位別集計表"

    # 『SS7』を起動して結果CSVファイルを作成
    ss7py.Init()
    ss7py.Start("1.1.1.19", 1)
    ss7data: ss7py.Ss7Data = ss7py.Open(data_path, 3, 3)
    n: int
    for n in range(1, 6):  # 結果1～5でループ
        res_name: str = f"結果{n}"
        dest_path: str = data_path + "\\" + f"cost{n}.csv"
        if os.path.isfile(dest_path):  # 既存の cost*.csv を削除
            os.remove(dest_path)
        res: ss7py.Ss7Result = ss7data.GetResultData(res_name)
        if res is not None:
            # 結果CSVファイル cost*.csv を作成
            res.ExportResultCsv(table_names, dest_path, 1, 1, 1)
    ss7data.Close(2)
    ss7py.End(2)

    # 作成した結果CSVファイルの情報を集計
    # ファイル作成の遅延を考慮して『SS7』終了後に行う。
    for n in range(1, 6):  # 結果1～5でループ
        path: str = data_path + "\\" + f"cost{n}.csv"
        if os.path.isfile(path):
            csv_paths.append(path)
            res_names.append(f"結果{n}")


def str2f(val: str) -> float:
    """
    文字列を実数に変換します。空文字のとき0.0にします。
    Note:
        float(空文字) はエラーになるのでこの関数を作りました。
    Args:
        val (str): 変換する文字列
    Returns:
        float: 変換後の値
    """
    if "" == val:
        return 0.0  # 空文字なら0.0
    else:
        return float(val)


def find_result(f: IO, res_name: str) -> bool:
    """
    結果CSVファイルから指定した解析結果を検索します。
    Note:
        見つかったとき、データの読み込み位置が「<data>」の直後に移動します。
    Args:
        f (IO): 結果CSVファイルのオブジェクト
        res_name (str): 解析結果の名前
    Returns:
        bool: True:見つかった False:見つからない
    """
    line: str = f.readline()
    while line:
        if res_name == line:
            line = f.readline()
            while line:
                if "<data>\n" == line:
                    return True  # "<data>"の行まで読み進めて終了
                line = f.readline()
        line = f.readline()
    return False


def read_csv_data(path: str, buf: list[float]) -> bool:
    """
    結果CSVファイルからデータを読み込みます。
    Args:
        path (str): 結果CSVファイルのパス名
        buf (list[float]): 読み込んだデータ
    Returns:
        bool: True=取得成功 False=未取得
    """
    res = False
    f: IO
    with open(path, "r", encoding="cp932") as f:
        if find_result(f, "name=部位別集計表(コンクリート，型枠，鉄筋)\n"):
            # name=XXXXが見つかったとき
            line: str = f.readline()
            while line:
                if "\n" == line:
                    break  # 空行なら終了
                # コンクリートの合計＝最初に見つかった"合計"を含む行
                if "合計" in line:
                    csv: list[str] = line.split(",")
                    buf[0] = str2f(csv[5])  # 柱
                    buf[1] = str2f(csv[6])  # 大梁
                    buf[2] = str2f(csv[9])  # 床
                    buf[3] = str2f(csv[10])  # 耐震壁
                    res = True
                    break
                line = f.readline()
    return res


""" メイン """
# 物件データのパス名
args: list = sys.argv  # コマンドライン引数リスト
argnum: int = len(args)  # コマンドライン引数の数
if 2 > argnum:
    print("物件データを指定していません")
    exit()
data_path: str = args[1]

# 制御データ
csv_paths: list[str] = []  # CSVパス名のリスト
res_names: list[str] = []  # 結果データ名のリスト

# 結果CSVファイルを作成, 制御データの更新
print("『SS7』起動")

create_res_csv()

print("『SS7』終了")

# 結果CSVファイルの読み込み
conc_qty: list[list[float]] = []  # コンクリート [結果1～5][柱,梁,床,耐震壁]
buf: list[float] = [0.0, 0.0, 0.0, 0.0]  # 読み込みバッファ
path: str  # 結果CSVファイルのパス名
print("結果CSVファイルの読み込み開始")

for path in csv_paths:
    if read_csv_data(path, buf):
        conc_qty.append([buf[0], buf[1], buf[2], buf[3]])
    os.remove(path)  # 用が済んだら削除

conc_col: int = 4  # hvalの列数 (j)
conc_raw: int = len(conc_qty)  # hvalの行数 (i)
if 0 == conc_raw:
    print("積算の解析結果が存在しません")
    exit()

print("結果CSVファイルの読み込み終了")

# グラフのパラメーター（数値はその場の思いつきで決めました）
f_widths: list[float] = [3, 3, 4, 5, 6]  # 結果数ごとの描画領域の幅
b_widths: list[float] = [0.875, 0.44, 0.29, 0.22, 0.17]  # 結果数ごとの棒の幅
b_colors: list[str] = ["b", "r", "g", "c", "m"]  # 棒の色 1番目, 2番目...

# 結果の数によって決まったパラメーター
f_width: float = f_widths[conc_raw - 1]  # 描画領域の幅
b_width: float = b_widths[conc_raw - 1]  # 棒の幅
m_counts: np.ndarray = np.arange(conc_col)  # 部位のカウント数リスト

# グラフの設定
caps: str = "コンクリート体積比較"  # ウィンドウのタイトル
mpl.rc("font", family="MS Gothic")  # 文字フォント
fig: mpl.figure.Figure = plt.figure(caps, figsize=(f_width, 6))  # 描画領域
ax: mpl.axes._axes.Axes = fig.add_subplot(1, 1, 1)  # サブプロット
ax.set_title("コンクリート体積[cm3]", fontsize=12)  # グラフのタイトル
i: int  # conc_qtyの要素インデックス
hval: list[float]  # conc_qtyの要素
for i, hval in enumerate(conc_qty):  # 棒グラフ
    ax.bar(
        m_counts + b_width * i,
        hval,
        color=b_colors[i],
        width=b_width,
        label=res_names[i],
    )

m_names: list[str] = ["柱", "大梁", "床", "耐震壁"]  # 部位名
plt.xticks(m_counts + b_width * (conc_raw - ((conc_raw + 1) / 2)), m_names, fontsize=10)  # X軸の目盛
plt.legend(res_names, loc="upper left", bbox_to_anchor=(1, 1), fontsize=10)  # 凡例

# 表示する
fig.tight_layout()  # ウィンドウ内に収める
plt.show()
